"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _helperPluginUtils = require("@babel/helper-plugin-utils");

var _core = require("@babel/core");

var _transformCreateElement = _interopRequireDefault(require("./transformCreateElement"));

var _cleverReplace = _interopRequireDefault(require("./cleverReplace"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _default = (0, _helperPluginUtils.declare)(api => {
  api.assertVersion(7);
  return {
    pre(state) {
      this.reactNamespace = new Set();
      this.directImported = new Map();
      this.globals = [];
      this.imports = [];

      this.getType = node => {
        if (_core.types.isIdentifier(node) && this.directImported.has(node.name)) {
          return this.directImported.get(node.name);
        } else if (_core.types.isMemberExpression(node) && _core.types.isIdentifier(node.object) && this.reactNamespace.has(node.object.name) && _core.types.isIdentifier(node.property)) {
          return node.property.name;
        }
      };

      const globalKeys = new Set();

      this.addGlobal = (node, key) => {
        if (key) {
          if (globalKeys.has(key)) return;
          globalKeys.add(key);
        }

        this.globals.push(node);
      };

      const importedHelpers = new Set();

      this.importHelper = exportName => {
        const name = `_rawact_${exportName}`;

        if (!importedHelpers.has(exportName)) {
          this.imports.push(_core.types.importDeclaration([_core.types.importDefaultSpecifier(_core.types.identifier(name))], _core.types.stringLiteral(`babel-plugin-rawact/lib/runtime/${exportName}`)));
          importedHelpers.add(exportName);
        }

        return _core.types.identifier(name);
      };

      let uniqueNumber = 0;

      this.getUniqueNumber = () => {
        return uniqueNumber++;
      };

      this.getUniqueIdentifier = name => {
        return state.path.scope.generateUidIdentifier(name);
      };
    },

    visitor: {
      ImportDeclaration(path) {
        if (!_core.types.isStringLiteral(path.node.source)) return;
        const source = path.node.source.value;
        if (source !== "react" && source !== "react-dom") return;
        const newSpecifiers = [];

        for (const specifier of path.node.specifiers) {
          if (_core.types.isImportDefaultSpecifier(specifier) || _core.types.isImportNamespaceSpecifier(specifier)) {
            this.reactNamespace.add(specifier.local.name);
            path.insertBefore(_core.types.importDeclaration([_core.types.importNamespaceSpecifier(specifier.local)], _core.types.stringLiteral("babel-plugin-rawact/lib/runtime/react")));
          }

          if (_core.types.isImportSpecifier(specifier)) {
            this.directImported.set(specifier.local.name, specifier.imported.name);
            newSpecifiers.push(_core.types.importSpecifier(specifier.local, specifier.imported));
          }
        }

        if (newSpecifiers.length > 0) {
          path.replaceWith(_core.types.importDeclaration(newSpecifiers, _core.types.stringLiteral("babel-plugin-rawact/lib/runtime/react")));
        } else {
          path.remove();
        }
      },

      Program: {
        exit(path) {
          path.traverse({
            CallExpression(path) {
              const callee = path.node.callee;
              const type = this.getType(callee);

              switch (type) {
                case "createElement":
                  {
                    (0, _cleverReplace.default)(path, (0, _transformCreateElement.default)(path.node, {
                      getType: this.getType.bind(this),
                      addGlobal: this.addGlobal.bind(this),
                      getUniqueIdentifier: this.getUniqueIdentifier.bind(this),
                      getUniqueNumber: this.getUniqueNumber.bind(this),
                      importHelper: this.importHelper.bind(this)
                    }));
                    break;
                  }
              }
            }

          }, this);
        }

      }
    },

    post(state) {
      for (const global of this.globals.reverse()) state.path.unshiftContainer("body", global);

      for (const imp of this.imports.reverse()) state.path.unshiftContainer("body", imp);
    }

  };
});

exports.default = _default;